<?php
/**
 * @file
 * Utility functions for Media: AP.
 */

/**
 * Based on the Photo ID of a AP image, this will return the URL to the image
 * itself.
 *
 * @param string $photo_id
 *   The AP photo code.
 * @param string $size
 *   Either 'full', to load the full size image (default), or 'thumbnail'.
 * @param boolean $cached
 *   If TRUE, then we'll cache the remote URL if the attempt to save the file
 *   locally fails.
 */
function _media_ap_photo_url($photo_id, $size = 'full', $cached = TRUE) {
  if (!empty($photo_id)) {
    return _media_ap_photo_url_from_size($photo_id, $size);
  }
}

/**
 * This returns a cached photo stream from the AP.
 *
 * @param string @photo_id
 * @param string $size
 *   Either 'full', to load the full size image (default), or 'thumbnail'.
 */
function _media_ap_photo_remote_url($photo_id, $size = 'full') {
  $photos = array();

  // Attempt to load the data from the cache.
  $cache = cache_get($photo_id, 'cache_media_ap');
  if (empty($cache) || empty($cache->data)) {
    // No data cached, so attempt loading the data from the API.
    $response = _media_ap_sets_request('item', array('photo_id' => $photo_id));
    $url_suffix = '&apiKey=' . check_plain(variable_get('media_ap_api_key'));

    foreach ($response['itemSet']['newsItem']['contentSet']['remoteContent'] as $size_id => $variant) {
      if (!empty($variant['@attributes']['href'])) {
        $response = drupal_http_request($variant['@attributes']['href'] . $url_suffix, array('max_redirects' => 0));
        if (!empty($response->redirect_url)) {
          $photos[$size_id] = $response->redirect_url;
        }
      }
    }
    cache_set($photo_id, $photos, 'cache_media_ap');
  }
  else {
    $photos = $cache->data;
  }

  // Load the full-size image.
  if ($size == 'full') {
    // Production mode uses the large image, otherwise use the preview.
    if (variable_get('media_ap_api_mode', 'production') == 'production') {
      $size = 0;
    }
    else {
      $size = 1;
    }
  }
  // Load the thumbnail.
  else {
    $size = 2;
  }

  return isset($photos[$size]) ? $photos[$size] : '';
}

/**
 * This is a wrapper for emfield_request_xml that includes AP's API key.
 *
 * @param string $method
 * @param array $args
 *
 * @return object
 *   A fully populated SimpleXML object.
 */
function _media_ap_sets_request($method, $args = array()) {
  // Display an error if we don't have an API key yet.
  if (!_media_ap_error_check()) {
    return array();
  }

  // Production vs Preview mode. This currently
  if (variable_get('media_ap_api_mode', 'production') == 'production') {
    $api_url = MEDIA_AP_REST_PROD;
  }
  else {
    $api_url = MEDIA_AP_REST_PREVIEW;
  }

  $api_url .= '/' . $method . '/' . $args['photo_id'];

  // Compile the full API request.
  $api_args = array(
    'apiKey' => check_plain(variable_get('media_ap_api_key', '')),
  );
  $api_url = url($api_url, array('query' => $api_args));

  $options = array(
    // Avoid sitting without a response for terribly long.
    'timeout' => 10,
    'headers' => array(
      'Accept' => 'application/json',
    ),
  );
  $request = drupal_http_request($api_url, $options);

  // Extract the response from the API.
  // if (!empty($request) && empty($request->error) && $request->code == 200) {
  //   $data = new SimpleXMLElement($request->data);
  // }
  // else {
  //   watchdog('media_ap', "Error querying the AP API. (Error: {$request->code}, {$request->error})");
  //   drupal_set_message("Error querying the AP API. (Error: {$request->code}, {$request->error})");
  //   drupal_set_message(t('Unable to access the AP API. Please try again later.'));
  // }

  if (!isset($request->error)) {
    $data = simplexml_load_string($request->data);
  }
  else {
    throw new Exception("Error Processing Request. (Error: {$request->code}, {$request->error}).");

    // If request wasn't successful, create object for return to avoid errors.
    $data = new SimpleXMLElement();
  }

  $xml = media_ap_unserialize_xml($data);
  return $xml;
}

/**
 * Recursively converts a SimpleXMLElement object into an array.
 *
 * @param object $xml
 *   The original XML object.
 */
function media_ap_unserialize_xml($xml) {
  if ($xml instanceof SimpleXMLElement) {
    $xml = (array) $xml;
  }
  if (is_array($xml)) {
    foreach ($xml as $key => $item) {
      $xml[$key] = media_ap_unserialize_xml($item);
    }
  }

  return $xml;
}

/**
 * This will log an error if we don't have a key yet.
 * In addition, if the user is an admin, we'll display an error.
 */
function _media_ap_error_check() {
  static $checked;
  if (is_null($checked)) {
    if (variable_get('media_ap_api_key', '') == '') {
      $error = 'You do not yet have a AP API key set. You will need to !apply and enter your key at the !settings before AP images may be displayed.';
      $arguments = array('!apply' => l(t('apply for an AP API key'), MEDIA_AP_API_APPLICATION_URL), '!settings' => l(t('AP administration page'), 'admin/config/media/ap'));
      if (user_access('administer site configuration')) {
        drupal_set_message(t($error, $arguments), 'error');
      }
      watchdog('media_ap', $error, $arguments);
      $checked = FALSE;
    }
    else {
      $checked = TRUE;
    }
  }
  return $checked;
}

function _media_ap_photo_url_from_size($photo_id, $size = 'full') {
  // Check to make sure we have a valid photo code.
  $filepath = _media_ap_photo_remote_url($photo_id, $size);

  if (empty($filepath)) {
    return FALSE;
  }
  else {
    return url($filepath, array('absolute' => TRUE));
  }
}

function _media_ap_original_url($uri, $refresh = FALSE) {
  $wrapper = file_stream_wrapper_get_instance_by_uri($uri);
  return $wrapper->interpolateUrl();
}

function _media_ap_photo_exists($photo_id) {
  $info = _media_ap_sets_request('item', array('photo_id' => $photo_id));

  if ($info && ($info['@attributes']['stat'] != 'fail')) {
    return TRUE;
  }
}

function _media_ap_build_uri($photo_id) {
  return file_stream_wrapper_uri_normalize('ap://' . $photo_id);
}

/**
 * Callback to parse a given URL and return a valid media_ap URL.
 *
 * @see media_parse_to_uri().
 */
function _media_ap_media_parse($url, $options = array()) {
  $patterns = array(
    // Example ID returned from the /search/photos API:
    // https://apipreview.ap.org/v2/item/7a9977aea6964faaaa80e5af805dedf5?sid=63c3e27b9690456bbdb28371456453a5
    '@ap\.org/v2/item/([0-9a-z]*)\?sid=(.*)@i',
    // ap://styles/[stylename]/ap/[photo_id]
    '@ap://(.*)/(.*)/ap/([0-9a-z]*)@i',
    // ap://[photo_id]
    '@ap://([0-9a-z]*)@i',
  );

  foreach ($patterns as $pattern) {
    preg_match($pattern, $url, $matches);
    if (isset($matches[3])) {
      return _media_ap_build_uri($matches[3]);
    }
    elseif (isset($matches[1])) {
      return _media_ap_build_uri($matches[1]);
    }
  }
}
